@@ -29,8 +29,7 @@ crost <- function(data,h=10,w=NULL,init=c("mean","naive"),nop=c(2,1),
 #                 "msr" - Mean squared rate
 #                 "mae" - Mean absolute error
 #                 "mse" - Mean squared error
-#   init.opt    If parameters are optimised and init.opt==TRUE then initial values are 
-#               optimised as well. 
+#   init.opt    If init.opt==TRUE then initial values are optimised. 
 #   outplot     If TRUE a plot of the forecast is provided.
 #   opt.on      This is meant to use only by the optimisation function. When opt.on is 
 #               TRUE then no checks on inputs are performed. 
@@ -70,6 +69,12 @@ crost <- function(data,h=10,w=NULL,init=c("mean","naive"),nop=c(2,1),
     }
   }
   
+  # Make sure that nop is of correct lenght
+  if (nop>2 || nop<1){
+    nop <- 2
+    warning("nop can be either 1 or 2. Overriden to 2.")
+  }
+  
   n <- length(data)
   
   # Croston decomposition
@@ -86,12 +91,14 @@ crost <- function(data,h=10,w=NULL,init=c("mean","naive"),nop=c(2,1),
       init <- c(z[1],x[1])
     }
   }
-  
+
   # Optimise parameters if requested
-  if (is.null(w)){
-    wopt <- crost.opt(data,type,cost,nop,init,init.opt)
-    w <- wopt$w
-    init <- wopt$init
+  if (opt.on == FALSE){
+    if (is.null(w) || init.opt == TRUE){
+      wopt <- crost.opt(data,type,cost,w,nop,init,init.opt)
+      w <- wopt$w
+      init <- wopt$init
+    }
   }
   
   # Pre-allocate memory
@@ -103,15 +110,13 @@ crost <- function(data,h=10,w=NULL,init=c("mean","naive"),nop=c(2,1),
   if (opt.on == FALSE){
     if (init[1]<0){
       stop("Initial demand cannot be a negative number.")
-    } else {
-      zfit[1] <- init[1]
-    }
+    } 
     if (init[2]<1){
       stop("Initial interval cannot be less than 1.")
-    } else {
-      xfit[1] <- init[2]
     }
-  }
+  } 
+  zfit[1] <- init[1]
+  xfit[1] <- init[2]
   
   if (length(w)==1){
     a.demand <- w[1]
@@ -135,7 +140,7 @@ crost <- function(data,h=10,w=NULL,init=c("mean","naive"),nop=c(2,1),
     zfit[i] <- zfit[i-1] + a.demand * (z[i] - zfit[i-1])      # Demand
     xfit[i] <- xfit[i-1] + a.interval * (x[i] - xfit[i-1])    # Interval
   }
-  c <- coeff * zfit/xfit
+  cc <- coeff * zfit/xfit
   
   # Calculate in-sample demand rate
   frc.in <- vector("numeric",n)
@@ -143,13 +148,13 @@ crost <- function(data,h=10,w=NULL,init=c("mean","naive"),nop=c(2,1),
   tv <- c(nzd+1,n)  # Time vector used to create frc.in forecasts
   for (i in 1:k){
     if (tv[i]<=n){
-      frc.in[tv[i]:min(c(tv[i+1],n))] <- c[i]
+      frc.in[tv[i]:min(c(tv[i+1],n))] <- cc[i]
     }
   }
   
   # Forecast out-of-sample demand rate
   if (h>0){
-    frc.out <- rep(c[k],h)
+    frc.out <- rep(cc[k],h)
   } else {
     frc.out = NULL
   }
@@ -175,7 +180,7 @@ crost <- function(data,h=10,w=NULL,init=c("mean","naive"),nop=c(2,1),
 
 #-------------------------------------------------
 crost.opt <- function(data,type=c("croston","sba","sbj"),cost=c("mar","msr","mae","mse"),
-                      nop=c(2,1),init,init.opt=c(TRUE,FALSE)){
+                      w=NULL,nop=c(2,1),init,init.opt=c(TRUE,FALSE)){
 # Optimisation function for Croston and variants
   
   type <- type[1]
@@ -188,22 +193,41 @@ crost.opt <- function(data,type=c("croston","sba","sbj"),cost=c("mar","msr","mae
   k <- length(nzd)
   x <- c(nzd[1],nzd[2:k]-nzd[1:(k-1)])  # Intervals
   
-  
-  if (init.opt == TRUE){
-    w <- c(rep(0.05,nop),init[1],init[2])
-    lbound <- c(rep(0,nop),0,1)
-    ubound <- c(rep(1,nop),max(data),max(x))
-    wopt <- optim(par=w,crost.cost,method="Nelder-Mead",data=data,cost=cost,
-                  type=type,nop=nop,init=init,init.opt=init.opt,lbound=lbound,ubound=ubound,
-                  control=list(maxit = 2000))$par
-  } else {
-    w <- c(rep(0.05,nop))
+  if (is.null(w) == TRUE && init.opt == FALSE){
+    # Optimise only w
+    p0 <- c(rep(0.05,nop))
     lbound <- c(rep(0,nop))
     ubound <- c(rep(1,nop))
-    wopt <- optim(par=w,crost.cost,method="Nelder-Mead",data=data,cost=cost,
-                  type=type,nop=nop,init=init,init.opt=init.opt,lbound=lbound,ubound=ubound,
-                  control=list(maxit = 2000))$par    
+    if (nop != 1){
+      wopt <- optim(par=p0,crost.cost,method="Nelder-Mead",data=data,cost=cost,
+                    type=type,w=w,nop=nop,w.opt=is.null(w),init=init,init.opt=init.opt,
+                    lbound=lbound,ubound=ubound,control=list(maxit=2000))$par    
+    } else {
+      # Use Brent
+      wopt <- optim(par=p0,crost.cost,method="Brent",data=data,cost=cost,
+                    type=type,w=w,nop=nop,w.opt=is.null(w),init=init,init.opt=init.opt,
+                    lbound=lbound,ubound=ubound,lower=lbound,upper=ubound,
+                    control=list(maxit=2000))$par  
+    }
     wopt <- c(wopt,init)
+  } else if (is.null(w) == TRUE && init.opt == TRUE){
+    # Optimise w and init
+    p0 <- c(rep(0.05,nop),init[1],init[2])
+    lbound <- c(rep(0,nop),0,1)
+    ubound <- c(rep(1,nop),max(data),max(x))
+    wopt <- optim(par=p0,crost.cost,method="Nelder-Mead",data=data,cost=cost,
+                  type=type,w=w,nop=nop,w.opt=is.null(w),init=init,init.opt=init.opt,
+                  lbound=lbound,ubound=ubound,control=list(maxit=2000))$par
+  } else if (is.null(w) == FALSE && init.opt == TRUE){
+    # Optimise only init
+    nop <- length(w)
+    p0 <- c(init[1],init[2])
+    lbound <- c(0,1)
+    ubound <- c(max(data),max(x))
+    wopt <- optim(par=p0,crost.cost,method="Nelder-Mead",data=data,cost=cost,
+                  type=type,w=w,nop=nop,w.opt=is.null(w),init=init,init.opt=init.opt,
+                  lbound=lbound,ubound=ubound,control=list(maxit=2000))$par
+    wopt <- c(w,wopt)
   }
   
   return(list(w=wopt[1:nop],init=wopt[(nop+1):(nop+2)]))
@@ -211,14 +235,18 @@ crost.opt <- function(data,type=c("croston","sba","sbj"),cost=c("mar","msr","mae
 }
 
 #-------------------------------------------------
-crost.cost <- function(w,data,cost,type,nop,init,init.opt,lbound,ubound){
+crost.cost <- function(p0,data,cost,type,w,nop,w.opt,init,init.opt,lbound,ubound){
 # Cost functions for Croston and variants  
   
-  if (init.opt==TRUE){
-    frc.in <- crost(data=data,w=w[1:nop],h=0,init=w[(nop+1):(nop+2)],
+  if (w.opt == TRUE && init.opt == TRUE){
+    frc.in <- crost(data=data,w=p0[1:nop],h=0,init=p0[(nop+1):(nop+2)],
+                    type=type,opt.on=TRUE)$frc.in
+  } else if (w.opt == TRUE && init.opt == FALSE){
+    frc.in <- crost(data=data,w=p0[1:nop],h=0,init=init,
+                    type=type,opt.on=TRUE)$frc.in
+  } else if (w.opt == FALSE && init.opt == TRUE){
+    frc.in <- crost(data=data,w=w,h=0,init=p0,
                     type=type,opt.on=TRUE)$frc.in
-  } else {
-    frc.in <- crost(data=data,w=w[1:nop],h=0,init=init,type=type,opt.on=TRUE)$frc.in
   }
   
   if (cost == "mse"){
@@ -248,8 +276,8 @@ crost.cost <- function(w,data,cost,type,nop,init,init.opt,lbound,ubound){
   }
   
   # Constrains
-  for (i in 1:(nop+2*init.opt)){
-    if (!(w[i]>=lbound[i]) | !(w[i]<=ubound[i])){
+  for (i in 1:(nop*w.opt+2*init.opt)){
+    if (!(p0[i]>=lbound[i]) | !(p0[i]<=ubound[i])){
       E <- 9*10^99
     }
   }
